/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_INCRENSCTLVECL95_H_
#define LORENZ95_INCRENSCTLVECL95_H_

#include <ostream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class GenericCtlVecL95;
  class LocalizationMatrixL95;
  class Resolution;

  /// IncrEnsCtlVecL95 Class
  /*!
   *  Augmented part of the control vector
   */

// -----------------------------------------------------------------------------
class IncrEnsCtlVecL95 : public util::Printable,
                         private util::ObjectCounter<IncrEnsCtlVecL95> {
 public:
  static const std::string classname() {return "lorenz95::IncrEnsCtlVecL95";}

/// Constructor, destructor
  IncrEnsCtlVecL95() {}
  IncrEnsCtlVecL95(const Resolution &, const LocalizationMatrixL95 &) {}
  IncrEnsCtlVecL95(const Resolution &, const LocalizationMatrixL95 &, const IncrEnsCtlVecL95 &) {}
  IncrEnsCtlVecL95(const IncrEnsCtlVecL95 &, const bool=true) {}
  virtual ~IncrEnsCtlVecL95() {}

/// Basic operators
  void zero() {}
  IncrEnsCtlVecL95 & operator =(const IncrEnsCtlVecL95 &) {return *this;} 
  IncrEnsCtlVecL95 & operator+=(const IncrEnsCtlVecL95 &) {return *this;}
  IncrEnsCtlVecL95 & operator-=(const IncrEnsCtlVecL95 &) {return *this;}
  IncrEnsCtlVecL95 & operator*=(const double &) {return *this;}
  void axpy(const double &, const IncrEnsCtlVecL95 &, const bool check = true) {};
  double dot_product_with(const IncrEnsCtlVecL95 &) const {return 0.0;}
  void schur_product_with(const IncrEnsCtlVecL95 &) {}
  void random() {}

// Utilities
  void read(const eckit::Configuration &) {}
  void write(const eckit::Configuration &) const {}
  double norm () const {return 0.0;}

/// Access to data
//  GenericCtlVecL95 & ctlvec() {return *ctlvec_;}
//  const GenericCtlVecL95 & ctlvec() const {return *ctlvec_;}  

 private:
  void print(std::ostream &) const {}
//  boost::scoped_ptr<GenericCtlVecL95> ctlvec_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_INCRENSCTLVECL95_H_
