/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_INCRMODCTLVECL95_H_
#define LORENZ95_INCRMODCTLVECL95_H_

#include <ostream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class ErrorCovarianceL95;
  class GenericCtlVecL95;
  class Resolution;

  /// IncrModCtlVecL95 Class
  /*!
   *  Increment part of the control vector (acted upon by the static background
   *  error covariance matrix)
   */

// -----------------------------------------------------------------------------
class IncrModCtlVecL95 : public util::Printable,
                         private util::ObjectCounter<IncrModCtlVecL95> {
 public:
  static const std::string classname() {return "lorenz95::IncrModCtlVecL95";}

/// Constructor, destructor
  IncrModCtlVecL95() {}
  IncrModCtlVecL95(const ErrorCovarianceL95 &) {}
  IncrModCtlVecL95(const ErrorCovarianceL95 &, const IncrModCtlVecL95 &) {}
  IncrModCtlVecL95(const IncrModCtlVecL95 &, const bool=true) {}
  virtual ~IncrModCtlVecL95() {}

/// Basic operators
  void zero() {}
  IncrModCtlVecL95 & operator =(const IncrModCtlVecL95 &) {return *this;} 
  IncrModCtlVecL95 & operator+=(const IncrModCtlVecL95 &) {return *this;}
  IncrModCtlVecL95 & operator-=(const IncrModCtlVecL95 &) {return *this;}
  IncrModCtlVecL95 & operator*=(const double &) {return *this;}
  void axpy(const double &, const IncrModCtlVecL95 &, const bool check = true) {};
  double dot_product_with(const IncrModCtlVecL95 &) const {return 0.0;}
  void schur_product_with(const IncrModCtlVecL95 &) {}
  void random() {}

/// Section
  void getSection(std::vector<double> &,
                  const size_t &,
                  const size_t &) const {}
  void setSection(const std::vector<double> &,
                  const size_t &,
                  const size_t &) {}

/// Utilities
  void read(const eckit::Configuration &) {}
  void write(const eckit::Configuration &) const {}
  double norm () const {return 0.0;}

/// Access to data
//  GenericCtlVecL95 & ctlvec() {return *ctlvec_;}
//  const GenericCtlVecL95 & ctlvec() const {return *ctlvec_;}  

 private:
  void print(std::ostream &) const {}
//  boost::scoped_ptr<GenericCtlVecL95> ctlvec_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_INCRMODCTLVECL95_H_
