/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_OBSBIASCORRECTION_H_
#define LORENZ95_OBSBIASCORRECTION_H_

#include <cmath>
#include <iostream>

#include "oops/interface/ObsAuxIncrementBase.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxControlBase;
}

namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class ObsBias;

// -----------------------------------------------------------------------------

class ObsBiasCorrection : public oops::ObsAuxIncrementBase {
 public:
/// Constructor, destructor
  explicit ObsBiasCorrection(const eckit::Configuration &);
  ObsBiasCorrection(const ObsBiasCorrection &, const bool copy = true);
  ObsBiasCorrection(const ObsBiasCorrection &, const eckit::Configuration &);
  virtual ~ObsBiasCorrection() {}

/// Cloning
  virtual ObsBiasCorrection* clone(const bool copy = true) const;
  virtual ObsBiasCorrection* clone(const eckit::Configuration &) const;

/// Linear algebra operators
  void diff(const oops::ObsAuxControlBase &, const oops::ObsAuxControlBase &);
  void zero();
  ObsBiasCorrection & operator=(const oops::ObsAuxIncrementBase &);
  ObsBiasCorrection & operator+=(const oops::ObsAuxIncrementBase &);
  ObsBiasCorrection & operator-=(const oops::ObsAuxIncrementBase &);
  ObsBiasCorrection & operator*=(const double &);
  void axpy(const double &, const oops::ObsAuxIncrementBase &);
  double dot_product_with(const oops::ObsAuxIncrementBase &) const;

/// I/O and diagnostics
  void read(const eckit::Configuration &) {}
  void write(const eckit::Configuration &) const {}
  double norm() const {return std::abs(bias_);}

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}

/// Local
  ObsBiasCorrection & operator=(const ObsBiasCorrection &);
  double & value() {return bias_;}
  const double & value() const {return bias_;}

 private:
  void print(std::ostream &) const;
  double bias_;
  bool active_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_OBSBIASCORRECTION_H_
