/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_OBSBIASCOVARIANCE_H_
#define LORENZ95_OBSBIASCOVARIANCE_H_

#include <ostream>
#include <string>
#include <boost/noncopyable.hpp>

#include "eckit/config/LocalConfiguration.h"
#include "oops/interface/ObsAuxCovarianceBase.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxControlBase;
  class ObsAuxCtlVecBase;
  class ObsAuxIncrementBase;
}

namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class ObsBias;
  class ObsBiasCtlVec;
  class ObsBiasCorrection;

// -----------------------------------------------------------------------------

class ObsBiasCovariance : public oops::ObsAuxCovarianceBase, 
                          private util::ObjectCounter<ObsBiasCovariance> {
 public:
  static const std::string classname() {return "lorenz95::ObsBiasCovariance";}

/// Constructor, destructor
  explicit ObsBiasCovariance(const eckit::Configuration &);
  virtual ~ObsBiasCovariance() {}

/// Update
  void update(const oops::ObsAuxControlBase &, const eckit::Configuration &) {}

/// Linear algebra operators
  void linearize(const oops::ObsAuxControlBase &) {}
  void multiply(const oops::ObsAuxIncrementBase &, oops::ObsAuxIncrementBase &) const;
  void inverseMultiply(const oops::ObsAuxIncrementBase &, oops::ObsAuxIncrementBase &) const;
  void multiplySqrt(const oops::ObsAuxCtlVecBase &, oops::ObsAuxIncrementBase &) const {}
  void multiplySqrtTrans(const oops::ObsAuxIncrementBase &, oops::ObsAuxCtlVecBase &) const {}
  void randomize(oops::ObsAuxIncrementBase &) const;

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}
  const eckit::Configuration & config() const {return conf_;}

/// Local
  bool active() const {return active_;}

 private:
  void print(std::ostream &) const;
  const eckit::LocalConfiguration conf_;
  double variance_;
  bool active_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_OBSBIASCOVARIANCE_H_
