/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_OBSTABLE_H_
#define LORENZ95_OBSTABLE_H_

#include <fstream>
#include <map>
#include <ostream>
#include <string>
#include <vector>

#include "oops/interface/ObsSpaceBase.h"
#include "util/DateTime.h"
#include "util/ObjectCounter.h"

namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class GomL95;
  class ObsVec1D;

/// A Simple Observation Data Handler
/*!
 *  ObsTable defines a simple observation handler
 *  that mimicks the interfaces required from ODB.
 */

// -----------------------------------------------------------------------------
class ObsTable : public oops::ObsSpaceBase,
                 private util::ObjectCounter<ObsTable> {
 public:
  static const std::string classname() {return "lorenz95::ObsTable";}

  enum Write : bool {
    Screened = true,
    Original = false
  };

  ObsTable(const eckit::Configuration &, 
           const util::DateTime &, 
           const util::DateTime &,
           const bool lscreened =false);
  ~ObsTable();

  void putdb(const std::string &, const std::vector<double> &) const;
  void getdb(const std::string &, std::vector<double> &) const;

  std::vector<double> locations(const util::DateTime &, const util::DateTime &) const;
  std::vector<int> timeSelect(const util::DateTime &, const util::DateTime &) const;
  void generateDistribution(const eckit::Configuration &);
  void printJo(const ObsVec1D &, const ObsVec1D &);
  unsigned int nobs() const {return times_.size();}
  void screenObservations(const ObsVec1D &, const GomL95 &) const;

 private:
  void print(std::ostream &) const;
  void otOpen(const std::string &);
  void otWrite(const std::string &, const bool &) const;

  const util::DateTime winbgn_;
  const util::DateTime winend_;
  const bool lscreened_;

  mutable std::vector<util::DateTime> times_;
  mutable std::vector<double> locations_;
  mutable std::map<std::string, std::vector<double> > data_;

  mutable std::vector<util::DateTime> screenedTimes_;
  mutable std::vector<double> screenedLocations_;
  mutable std::map<std::string, std::vector<double> > screenedData_;

  std::string nameIn_;
  std::string nameOut_;
};
// -----------------------------------------------------------------------------
}  // namespace lorenz95

#endif  // LORENZ95_OBSTABLE_H_
