/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef LORENZ95_OBSERVATIONTLAD_H_
#define LORENZ95_OBSERVATIONTLAD_H_

#include <ostream>
#include <string>

#include <boost/shared_ptr.hpp>

#include "oops/interface/LinearObsOperBase.h"
#include "util/ObjectCounter.h"
#include "lorenz95/L95Traits.h"

// Forward declarations
namespace eckit {
  class Configuration;
}

namespace lorenz95 {
  class GomL95;
  class ObsBias;
  class ObsBiasCorrection;
  class ObsTable;
  class ObsVec1D;
  class NoVariables;

/// Observation for Lorenz 95 model.
/*!
 *  ObservationTLAD defines the linearized ObsOperator for Lorenz 95 model.
 */

// -----------------------------------------------------------------------------

class ObservationTLAD : public oops::LinearObsOperBase<L95Traits>,
                        private util::ObjectCounter<ObservationTLAD> {
 public:
  static const std::string classname() {return "lorenz95::ObservationTLAD";}

  ObservationTLAD(const ObsTable &, const eckit::Configuration &);
  ~ObservationTLAD();

// Obs Operators
  void setTrajectory(const GomL95 &, 
                     const boost::ptr_map<std::string, oops::ObsAuxControlBase> &);
  void obsEquivTL(const GomL95 &, ObsVec1D &, 
                  const boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const;
  void obsEquivAD(GomL95 &, const ObsVec1D &, 
                  boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const;
  void obsBiasEquivTL(const GomL95 &, ObsVec1D &, 
                      const boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const {}
  void obsBiasEquivAD(const GomL95 &, const ObsVec1D &, 
                      boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const {}

// Other
  boost::shared_ptr<const NoVariables> variables() const {return inputs_;}

 private:
  void print(std::ostream &) const;
  boost::shared_ptr<const NoVariables> inputs_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // LORENZ95_OBSERVATIONTLAD_H_
