/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_GENERICCTLVECQG_H_
#define QG_MODEL_GENERICCTLVECQG_H_

#include <ostream>
#include <string>
#include <vector>

#include <boost/shared_ptr.hpp>

#include "model/QgFortran.h"

#include "util/ObjectCounter.h"
#include "util/Printable.h"

// Forward declarations
namespace eckit {
  class Configuration;
}

namespace qg {

  /// GenericCtlVecQG Class
  /*!
   * Generic class for handling control vector components storage and operations
   */

// -----------------------------------------------------------------------------
class GenericCtlVecQG : public util::Printable,
                        private util::ObjectCounter<GenericCtlVecQG> {
 public:
  static const std::string classname() {return "qg::GenericCtlVecQG";}

// Constructors and basic operators
  GenericCtlVecQG(const int & nx, const int & ny, const int & nl);
  GenericCtlVecQG(const GenericCtlVecQG &, const bool);
  GenericCtlVecQG(const GenericCtlVecQG &);
  ~GenericCtlVecQG();

  void zero();
  GenericCtlVecQG & operator=(const GenericCtlVecQG &);
  GenericCtlVecQG & operator+=(const GenericCtlVecQG &);
  GenericCtlVecQG & operator-=(const GenericCtlVecQG &);
  GenericCtlVecQG & operator*=(const double &);
  void axpy(const double &, const GenericCtlVecQG &);
  double dot_product_with(const GenericCtlVecQG &) const;
  void schur_product_with(const GenericCtlVecQG &);
  void random();

// Interpolate full fields
  void changeResolution(const GenericCtlVecQG &);

// Section
  void getSection(std::vector<double> &,
                  const int &,
                  const int &) const;
  void setSection(const std::vector<double> &,
                  const int &,
                  const int &);

// Utilities
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

  int & toFortran() {return keyCtlVec_;}
  const int & toFortran() const {return keyCtlVec_;}

 private:
  void print(std::ostream &) const;
  int nx_;
  int ny_;
  int nl_;
  F90ctlvec keyCtlVec_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_GENERICCTLVECQG_H_
