/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/IncrEnsCtlVecQG.h"

#include <algorithm>
#include <string>

#include "model/LocalizationMatrixQG.h"
#include "model/GenericCtlVecQG.h"
#include "model/GeometryQG.h"

#include "eckit/config/LocalConfiguration.h"
#include "util/DateTime.h"
#include "util/Logger.h"

using oops::Log;

namespace qg {

// -----------------------------------------------------------------------------
/// Constructor, destructor
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG::IncrEnsCtlVecQG (const GeometryQG & resol, 
                                  const LocalizationMatrixQG & loc)
  : ctlvec_()
{
  std::vector<int> dim(loc.getDim());
  ctlvec_.reset(new GenericCtlVecQG(dim[0], dim[1], dim[2]));
  ctlvec_->zero();
  Log::trace() << "IncrEnsCtlVecQG constructed." << std::endl;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG::IncrEnsCtlVecQG(const GeometryQG & resol, 
                                 const LocalizationMatrixQG & loc,
                                 const IncrEnsCtlVecQG & other)
  : ctlvec_()
{
  std::vector<int> dim(loc.getDim());
  ctlvec_.reset(new GenericCtlVecQG(dim[0], dim[1], dim[2]));
  *ctlvec_ = *other.ctlvec_;
  Log::trace() << "IncrEnsCtlVecQG constructed from other." << std::endl;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG::IncrEnsCtlVecQG(const IncrEnsCtlVecQG & other, const bool copy)
  : ctlvec_(new GenericCtlVecQG(*other.ctlvec_, copy))
{
  Log::trace() << "IncrEnsCtlVecQG copy-created." << std::endl;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG::~IncrEnsCtlVecQG() {
  ctlvec_.reset();
  Log::trace() << "IncrEnsCtlVecQG destructed" << std::endl;
}
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
/// Basic operators
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG & IncrEnsCtlVecQG::operator=(const IncrEnsCtlVecQG & rhs) {
  *ctlvec_ = *rhs.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG & IncrEnsCtlVecQG::operator+=(const IncrEnsCtlVecQG & dx) {
  *ctlvec_ += *dx.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG & IncrEnsCtlVecQG::operator-=(const IncrEnsCtlVecQG & dx) {
  *ctlvec_ -= *dx.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrEnsCtlVecQG & IncrEnsCtlVecQG::operator*=(const double & zz) {
  *ctlvec_ *= zz;
  return *this;
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::zero() {
  ctlvec_->zero();
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::axpy(const double & zz, const IncrEnsCtlVecQG & dx,
                       const bool check) {
  ctlvec_->axpy(zz, *dx.ctlvec_);
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::schur_product_with(const IncrEnsCtlVecQG & dx) {
  ctlvec_->schur_product_with(*dx.ctlvec_);
}
// -----------------------------------------------------------------------------
double IncrEnsCtlVecQG::dot_product_with(const IncrEnsCtlVecQG & other) const {
  return dot_product(*ctlvec_, *other.ctlvec_);
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::random() {
  ctlvec_->random();
}
// -----------------------------------------------------------------------------
/// I/O and diagnostics
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::read(const eckit::Configuration & files) {
  ctlvec_->read(files);
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::write(const eckit::Configuration & files) const {
  ctlvec_->write(files);
}
// -----------------------------------------------------------------------------
void IncrEnsCtlVecQG::print(std::ostream & os) const {
  os << *ctlvec_;
}
// -----------------------------------------------------------------------------

}  // namespace qg
