/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_INCREnsCtlVECQG_H_
#define QG_MODEL_INCREnsCtlVECQG_H_

#include <ostream>
#include <string>

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include "model/GenericCtlVecQG.h"
#include "model/GeometryQG.h"

#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"
#include "util/dot_product.h"

namespace eckit {
  class Configuration;
}

namespace qg {
  class LocalizationMatrixQG;

  /// IncrEnsCtlVecQG Class
  /*!
   *  Augmented part of the control vector
   */

// -----------------------------------------------------------------------------

class IncrEnsCtlVecQG : public util::Printable,
                        private util::ObjectCounter<IncrEnsCtlVecQG> {
 public:
  static const std::string classname() {return "qg::IncrEnsCtlVecQG";}

/// Constructor, destructor
  IncrEnsCtlVecQG();  
  IncrEnsCtlVecQG(const GeometryQG &, const LocalizationMatrixQG &);  
  IncrEnsCtlVecQG(const GeometryQG &, const LocalizationMatrixQG &, const IncrEnsCtlVecQG &);  
  IncrEnsCtlVecQG(const IncrEnsCtlVecQG &, const bool);
  virtual ~IncrEnsCtlVecQG();

/// Basic operators
  void zero();
  //void zero(const util::DateTime &);
  IncrEnsCtlVecQG & operator =(const IncrEnsCtlVecQG &);
  IncrEnsCtlVecQG & operator+=(const IncrEnsCtlVecQG &);
  IncrEnsCtlVecQG & operator-=(const IncrEnsCtlVecQG &);
  IncrEnsCtlVecQG & operator*=(const double &);
  void axpy(const double &, const IncrEnsCtlVecQG &, const bool check = true);
  double dot_product_with(const IncrEnsCtlVecQG &) const;
  void schur_product_with(const IncrEnsCtlVecQG &);
  void random();

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const; 
  double norm() const;

/// Access to ctlvec
  GenericCtlVecQG & ctlvec() {return *ctlvec_;}
  const GenericCtlVecQG & ctlvec() const {return *ctlvec_;}

/// Data
 private:
  void print(std::ostream &) const;
  boost::scoped_ptr<GenericCtlVecQG> ctlvec_;
};
// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_INCRENSCTLVECQG_H_
