/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/IncrModCtlVecQG.h"

#include <algorithm>
#include <string>

#include "model/ErrorCovarianceQG.h"
#include "model/GenericCtlVecQG.h"

#include "eckit/config/LocalConfiguration.h"
#include "util/DateTime.h"
#include "util/Logger.h"

using oops::Log;

namespace qg {

// -----------------------------------------------------------------------------
/// Constructor, destructor
// -----------------------------------------------------------------------------
IncrModCtlVecQG::IncrModCtlVecQG (const ErrorCovarianceQG & cov)
  : ctlvec_()
{
  std::vector<int> dim(cov.getDim());
  ctlvec_.reset(new GenericCtlVecQG(dim[0], dim[1], dim[2]));
  ctlvec_->zero();
  Log::trace() << "IncrModCtlVecQG constructed." << std::endl;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG::IncrModCtlVecQG(const ErrorCovarianceQG & cov,
                                 const IncrModCtlVecQG & other)
  : ctlvec_()
{
  std::vector<int> dim(cov.getDim());
  ctlvec_.reset(new GenericCtlVecQG(dim[0], dim[1], dim[2]));
  *ctlvec_ = *other.ctlvec_;
  Log::trace() << "IncrModCtlVecQG constructed from other." << std::endl;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG::IncrModCtlVecQG(const IncrModCtlVecQG & other, const bool copy)
  : ctlvec_(new GenericCtlVecQG(*other.ctlvec_, copy))
{
  Log::trace() << "IncrModCtlVecQG copy-created." << std::endl;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG::~IncrModCtlVecQG() {
  ctlvec_.reset();
  Log::trace() << "IncrModCtlVecQG destructed" << std::endl;
}
// -----------------------------------------------------------------------------
/// Basic operators
// -----------------------------------------------------------------------------
IncrModCtlVecQG & IncrModCtlVecQG::operator=(const IncrModCtlVecQG & rhs) {
  *ctlvec_ = *rhs.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG & IncrModCtlVecQG::operator+=(const IncrModCtlVecQG & dx) {
  *ctlvec_ += *dx.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG & IncrModCtlVecQG::operator-=(const IncrModCtlVecQG & dx) {
  *ctlvec_ -= *dx.ctlvec_;
  return *this;
}
// -----------------------------------------------------------------------------
IncrModCtlVecQG & IncrModCtlVecQG::operator*=(const double & zz) {
  *ctlvec_ *= zz;
  return *this;
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::zero() {
  ctlvec_->zero();
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::axpy(const double & zz, const IncrModCtlVecQG & dx,
                       const bool check) {
  ctlvec_->axpy(zz, *dx.ctlvec_);
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::schur_product_with(const IncrModCtlVecQG & dx) {
  ctlvec_->schur_product_with(*dx.ctlvec_);
}
// -----------------------------------------------------------------------------
double IncrModCtlVecQG::dot_product_with(const IncrModCtlVecQG & other) const {
  return dot_product(*ctlvec_, *other.ctlvec_);
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::random() {
  ctlvec_->random();
}

// -----------------------------------------------------------------------------
/// Section
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::getSection(std::vector<double> & section,
                                 const int & iistart,
                                 const int & iiend) const {
  ctlvec_->getSection(section, iistart, iiend);
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::setSection(const std::vector<double> & section,
                                 const int & iistart,
                                 const int & iiend) {
  ctlvec_->setSection(section, iistart, iiend);
}
// -----------------------------------------------------------------------------
/// I/O and diagnostics
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::read(const eckit::Configuration & files) {
  ctlvec_->read(files);
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::write(const eckit::Configuration & files) const {
  ctlvec_->write(files);
}
// -----------------------------------------------------------------------------
void IncrModCtlVecQG::print(std::ostream & os) const {
  os << *ctlvec_;
}
// -----------------------------------------------------------------------------

}  // namespace qg
