/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_INCRMODCTLVECQG_H_
#define QG_MODEL_INCRMODCTLVECQG_H_

#include <ostream>
#include <string>

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include "model/GenericCtlVecQG.h"

#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"
#include "util/dot_product.h"

namespace eckit {
  class Configuration;
}

namespace qg {
  class ErrorCovarianceQG;

  /// IncrModCtlVecQG Class
  /*!
   *  Increment part of the control vector (acted upon by the static background
   *  error covariance matrix)
   */

// -----------------------------------------------------------------------------

class IncrModCtlVecQG : public util::Printable,
                        private util::ObjectCounter<IncrModCtlVecQG> {
 public:
  static const std::string classname() {return "qg::IncrModCtlVecQG";}

/// Constructor, destructor
  IncrModCtlVecQG();  
  IncrModCtlVecQG(const ErrorCovarianceQG &);
  IncrModCtlVecQG(const ErrorCovarianceQG &, const IncrModCtlVecQG &);
  IncrModCtlVecQG(const IncrModCtlVecQG &, const bool=true);
  virtual ~IncrModCtlVecQG();

/// Basic operators
  void zero();
  IncrModCtlVecQG & operator =(const IncrModCtlVecQG &);
  IncrModCtlVecQG & operator+=(const IncrModCtlVecQG &);
  IncrModCtlVecQG & operator-=(const IncrModCtlVecQG &);
  IncrModCtlVecQG & operator*=(const double &);
  void axpy(const double &, const IncrModCtlVecQG &, const bool check = true);
  double dot_product_with(const IncrModCtlVecQG &) const;
  void schur_product_with(const IncrModCtlVecQG &);
  void random();

/// Section
 void getSection(std::vector<double> &,
                 const int &,
                 const int &) const;
 void setSection(const std::vector<double> &,
                 const int &,
                 const int &);

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const {return ctlvec_->norm();}

/// Access to ctlvec
  GenericCtlVecQG & ctlvec() {return *ctlvec_;}
  const GenericCtlVecQG & ctlvec() const {return *ctlvec_;}

/// Data
 private:
  void print(std::ostream &) const;
  boost::scoped_ptr<GenericCtlVecQG> ctlvec_;
};
// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_INCRMODCTLVECQG_H_
