/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/LocalizationMatrixQG.h"

#include "model/GeometryQG.h"
#include "model/IncrementQG.h"
#include "model/IncrEnsCtlVecQG.h"

#include "eckit/config/Configuration.h"

// -----------------------------------------------------------------------------
namespace qg {
// -----------------------------------------------------------------------------
LocalizationMatrixQG::LocalizationMatrixQG(const GeometryQG & resol,
                                           const eckit::Configuration & config) {
  const eckit::Configuration * configc = &config;
  qg_localization_setup_f90(keyFtnConfig_, &configc, resol.toFortran());
}
// -----------------------------------------------------------------------------
LocalizationMatrixQG::~LocalizationMatrixQG() {
  qg_localization_delete_f90(keyFtnConfig_);
}
// -----------------------------------------------------------------------------
void LocalizationMatrixQG::multiply(IncrementQG & dx) const {
  qg_localization_mult_f90(keyFtnConfig_, dx.fields().toFortran());
}
// -----------------------------------------------------------------------------
void LocalizationMatrixQG::multiplySqrt(const IncrEnsCtlVecQG & dv, IncrementQG & dx) const {
  qg_localization_mult_sqrt_f90(keyFtnConfig_, dv.ctlvec().toFortran(),
                                               dx.fields().toFortran());
}
// -----------------------------------------------------------------------------
void LocalizationMatrixQG::multiplySqrtTrans(const IncrementQG & dx, IncrEnsCtlVecQG & dv) const {
  qg_localization_mult_sqrt_trans_f90(keyFtnConfig_, dx.fields().toFortran(),
                                                     dv.ctlvec().toFortran());
}
// -----------------------------------------------------------------------------
const std::vector<int> LocalizationMatrixQG::getDim() const {
  std::vector<int> dim(3);
  qg_localization_get_dim_f90(keyFtnConfig_, dim[0]);
  return dim;
}
// -----------------------------------------------------------------------------
void LocalizationMatrixQG::print(std::ostream & os) const {
  os << "LocalizationMatrixQG::print not implemented";
}
// -----------------------------------------------------------------------------

}  // namespace qg
