/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_MODELBIAS_H_
#define QG_MODEL_MODELBIAS_H_

#include <iostream>
#include <string>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "model/FieldsQG.h"

#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace eckit {
  class Configuration;
}

namespace qg {
  class GeometryQG;
  class ModelBiasIncrement;

/// Model error for the QG model.
/*!
 * This class is used to manipulate parameters of the model that
 * can be estimated in the assimilation. This includes model bias for
 * example but could be used for other parameters to be estimated.
 * This is sometimes referred to as augmented state or augmented
 * control variable in the litterature.
 * The augmented state is understood here as an augmented 4D state.
 */

// -----------------------------------------------------------------------------

class ModelBias : public util::Printable,
                  private boost::noncopyable,
                  private util::ObjectCounter<ModelBias> {
 public:
  static const std::string classname() {return "qg::ModelBias";}

  ModelBias(const GeometryQG &, const eckit::Configuration &);
  ModelBias(const GeometryQG &, const ModelBias &);
  ModelBias(const ModelBias &, const bool);
  ~ModelBias();

  ModelBias & operator+=(const ModelBiasIncrement &);

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

/// Access to fields
  FieldsQG & fields() {return *fields_;}
  const FieldsQG & fields() const {return *fields_;}

/// Status
  const bool & isActive() const {return active_;}

 private:
  void print(std::ostream & os) const;
  bool active_;
  boost::scoped_ptr<FieldsQG> fields_;
  boost::scoped_ptr<FieldsQG> stash_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_MODELBIAS_H_
