/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_MODELBIASCOVARIANCE_H_
#define QG_MODEL_MODELBIASCOVARIANCE_H_

#include <ostream>
#include <string>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "model/QgFortran.h"

#include "eckit/config/LocalConfiguration.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace qg {
  class ModelBias;
  class ModelBiasCtlVec;
  class ModelBiasIncrement;
  class GeometryQG;

// -----------------------------------------------------------------------------

class ModelBiasCovariance : public util::Printable,
                            private boost::noncopyable,
                            private util::ObjectCounter<ModelBiasCovariance> {
 public:
  static const std::string classname() {return "qg::ModelBiasCovariance";}

/// Constructor, destructor
  ModelBiasCovariance(const eckit::Configuration &, const GeometryQG &);
  ~ModelBiasCovariance();

/// Linear algebra operators
  void linearize(const ModelBias &, const GeometryQG &);
  void multiply(const ModelBiasIncrement &, ModelBiasIncrement &) const;
  void inverseMultiply(const ModelBiasIncrement &, ModelBiasIncrement &) const;
  void multiplySqrt(const ModelBiasCtlVec &, ModelBiasIncrement &) const;
  void multiplySqrtTrans(const ModelBiasIncrement &, ModelBiasCtlVec &) const;
  void randomize(ModelBiasIncrement &) const;

  const eckit::Configuration & config() const {return conf_;}

// Status
  const bool & isActive() const {return active_;}
 
// Get dimensions for control vector
  const std::vector<int> getDim() const;
 private:
  void print(std::ostream & os) const;
  bool active_;
  const eckit::LocalConfiguration conf_;
  boost::scoped_ptr<const GeometryQG> geom_;

  F90bmat keyBiasCov_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_MODELBIASCOVARIANCE_H_
