/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_MODELBIASCTLVEC_H_
#define QG_MODEL_MODELBIASCTLVEC_H_

#include <iostream>
#include <vector>

#include <boost/scoped_ptr.hpp>

#include "model/GenericCtlVecQG.h"

#include "util/Printable.h"

namespace eckit {
  class Configuration;
}

namespace qg {
  class ModelBias;
  class ModelBiasCovariance;

  /// ModelBiasCtlVec Class
  /*!
   *  Part of the control vector corresponding to the ModelAuxIncrement part of the
   *  control increment
   */

// -----------------------------------------------------------------------------

class ModelBiasCtlVec : public util::Printable {
 public:
/// Constructor, destructor
  ModelBiasCtlVec(const ModelBiasCovariance &);
  ModelBiasCtlVec(const ModelBiasCovariance &, const ModelBiasCtlVec &);
  ModelBiasCtlVec(const ModelBiasCtlVec &, const bool copy = true);
  ~ModelBiasCtlVec();

/// Linear algebra operators
  void zero();
  ModelBiasCtlVec & operator=(const ModelBiasCtlVec &);
  ModelBiasCtlVec & operator+=(const ModelBiasCtlVec &);
  ModelBiasCtlVec & operator-=(const ModelBiasCtlVec &);
  ModelBiasCtlVec & operator*=(const double &);
  void axpy(const double &, const ModelBiasCtlVec &);
  double dot_product_with(const ModelBiasCtlVec &) const;

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

/// Access to ctlvec
  GenericCtlVecQG & ctlvec() {return *ctlvec_;}
  const GenericCtlVecQG & ctlvec() const {return *ctlvec_;}

/// Status  
  const bool & isActive() const {return active_;}

 private:
  void print(std::ostream &) const;
  bool active_;

  boost::scoped_ptr<GenericCtlVecQG> ctlvec_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_MODELBIASCTLVEC_H_
