/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSBIAS_H_
#define QG_MODEL_OBSBIAS_H_

#include <iostream>
#include <string>
#include <vector>
#include <boost/noncopyable.hpp>

#include "oops/interface/ObsAuxControlBase.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxIncrementBase;
}

namespace eckit {
  class Configuration;
}

namespace qg {

/// Class to handle observation bias parameters.

// -----------------------------------------------------------------------------

class ObsBias : public oops::ObsAuxControlBase,
                private boost::noncopyable,
                private util::ObjectCounter<ObsBias> {
 public:
  static const unsigned int ntypes = 4;
  static const std::string classname() {return "qg::ObsBias";}

/// Constructor, destructor
  explicit ObsBias(const eckit::Configuration &);
  ObsBias(const ObsBias &, const bool);
  virtual ~ObsBias() {}

/// Cloning
  virtual ObsBias* clone(const bool) const;

/// Operators
  oops::ObsAuxControlBase & operator+=(const oops::ObsAuxIncrementBase &);

/// Reset 
  void reset(const std::string & stateid,
             const eckit::Configuration & conf);

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}

/// Local
  const double & operator[](const unsigned int ii) const {return bias_[ii];}

  const double & stream() const {return bias_[0];}
  const double & wind() const {return bias_[1];}
  const double & wspd() const {return bias_[3];}

 private:
  void print(std::ostream &) const;

  bool active_;
  std::vector<double> bias_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_OBSBIAS_H_
