/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSBIASCOVARIANCE_H_
#define QG_MODEL_OBSBIASCOVARIANCE_H_

#include <ostream>
#include <string>
#include <vector>
#include <boost/noncopyable.hpp>

#include "eckit/config/LocalConfiguration.h"
#include "oops/interface/ObsAuxCovarianceBase.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxControlBase;
  class ObsAuxCtlVecBase;
  class ObsAuxIncrementBase;
}

namespace qg {

// -----------------------------------------------------------------------------

class ObsBiasCovariance : public oops::ObsAuxCovarianceBase,
                          private util::ObjectCounter<ObsBiasCovariance> {
 public:
  static const std::string classname() {return "qg::ObsBiasCovariance";}

/// Constructor, destructor
  explicit ObsBiasCovariance(const eckit::Configuration &);
  virtual ~ObsBiasCovariance() {}

/// Update
  void update(const oops::ObsAuxControlBase &, const eckit::Configuration &) {}

/// Linear algebra operators
  void linearize(const oops::ObsAuxControlBase &) {}
  void multiply(const oops::ObsAuxIncrementBase &, oops::ObsAuxIncrementBase &) const;
  void inverseMultiply(const oops::ObsAuxIncrementBase &, oops::ObsAuxIncrementBase &) const;
  void multiplySqrt(const oops::ObsAuxCtlVecBase &, oops::ObsAuxIncrementBase &) const;
  void multiplySqrtTrans(const oops::ObsAuxIncrementBase &, oops::ObsAuxCtlVecBase &) const;
  void randomize(oops::ObsAuxIncrementBase &) const;

  bool active(const unsigned int ii) const {return variance_[ii] > 0.0;}

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}
  const eckit::Configuration & config() const {return conf_;}

 private:
  void print(std::ostream &) const;
  const eckit::LocalConfiguration conf_;
  std::vector<double> variance_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_OBSBIASCOVARIANCE_H_
