/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSBIASCTLVEC_H_
#define QG_MODEL_OBSBIASCTLVEC_H_

#include <iostream>
#include <vector>

#include "oops/interface/ObsAuxCtlVecBase.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxCovarianceBase;
}

namespace eckit {
  class Configuration;
}

namespace qg {

  /// ObsBiasCtlVec Class
  /*!
   *  Part of the control vector corresponding to the ObsAuxIncrement part of the
   *  control increment
   */

// -----------------------------------------------------------------------------

class ObsBiasCtlVec : public oops::ObsAuxCtlVecBase {
 public:
/// Constructor, destructor
  ObsBiasCtlVec(const oops::ObsAuxCovarianceBase &);
  ObsBiasCtlVec(const oops::ObsAuxCovarianceBase &, const ObsBiasCtlVec &);
  ObsBiasCtlVec(const ObsBiasCtlVec &, const bool copy = true);
  virtual ~ObsBiasCtlVec() {sqrtbias_.clear(); active_.clear();}

/// Cloning
  virtual ObsBiasCtlVec* clone(const oops::ObsAuxCovarianceBase &) const;
  virtual ObsBiasCtlVec* clone(const bool copy = true) const;

/// Linear algebra operators
  void zero();
  ObsBiasCtlVec & operator=(const oops::ObsAuxCtlVecBase &);
  ObsBiasCtlVec & operator+=(const oops::ObsAuxCtlVecBase &);
  ObsBiasCtlVec & operator-=(const oops::ObsAuxCtlVecBase &);
  ObsBiasCtlVec & operator*=(const double &);
  void axpy(const double &, const oops::ObsAuxCtlVecBase &);
  double dot_product_with(const oops::ObsAuxCtlVecBase &) const;

/// I/O and diagnostics
  void read(const eckit::Configuration &);
  void write(const eckit::Configuration &) const;
  double norm() const;

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}

/// Local
  double & operator[](const unsigned int ii) {return sqrtbias_[ii];}
  const double & operator[](const unsigned int ii) const {return sqrtbias_[ii];}

  double & stream() {return sqrtbias_[0];}
  double & wind() {return sqrtbias_[1];}
  double & wspd() {return sqrtbias_[3];}
  const double & stream() const {return sqrtbias_[0];}
  const double & wind() const {return sqrtbias_[1];}
  const double & wspd() const {return sqrtbias_[3];}

  const std::vector<bool> & active() const {return active_;} 

 private:
  void print(std::ostream &) const;
  void makePassive();

  std::vector<bool> active_;
  std::vector<double> sqrtbias_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_OBSBIASCTLVEC_H_
