/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSBIASINCREMENT_H_
#define QG_MODEL_OBSBIASINCREMENT_H_

#include <iostream>
#include <vector>

#include "oops/interface/ObsAuxIncrementBase.h"
#include "util/Printable.h"

namespace oops {
  class ObsAuxControlBase;
}

namespace eckit {
  class Configuration;
}

namespace qg {

// -----------------------------------------------------------------------------

class ObsBiasIncrement : public oops::ObsAuxIncrementBase {
 public:
/// Constructor, destructor
  explicit ObsBiasIncrement(const eckit::Configuration &);
  ObsBiasIncrement(const ObsBiasIncrement & other, const bool copy);
  ObsBiasIncrement(const ObsBiasIncrement & other, const eckit::Configuration &);
  virtual ~ObsBiasIncrement() {}

/// Cloning
  virtual ObsBiasIncrement* clone(const bool copy = true) const;
  virtual ObsBiasIncrement* clone(const eckit::Configuration &) const;

/// Linear algebra operators
  void diff(const oops::ObsAuxControlBase &, const oops::ObsAuxControlBase &);
  void zero();
  ObsBiasIncrement & operator=(const oops::ObsAuxIncrementBase &);
  ObsBiasIncrement & operator+=(const oops::ObsAuxIncrementBase &);
  ObsBiasIncrement & operator-=(const oops::ObsAuxIncrementBase &);
  ObsBiasIncrement & operator*=(const double &);
  void axpy(const double &, const oops::ObsAuxIncrementBase &);
  double dot_product_with(const oops::ObsAuxIncrementBase &) const;

/// I/O and diagnostics
  void read(const eckit::Configuration &) {}
  void write(const eckit::Configuration &) const {}
  double norm() const;

/// Interfacing
  const int & getAccess() const {return dummyKey_;}
  int & getAccess() {return dummyKey_;}

/// Local
  double & operator[](const unsigned int ii) {return bias_[ii];}
  const double & operator[](const unsigned int ii) const {return bias_[ii];}

  double & stream() {return bias_[0];}
  double & wind() {return bias_[1];}
  double & wspd() {return bias_[3];}
  const double & stream() const {return bias_[0];}
  const double & wind() const {return bias_[1];}
  const double & wspd() const {return bias_[3];}

 private:
  void print(std::ostream &) const;
  void makePassive();

  std::vector<bool> active_;
  std::vector<double> bias_;
  int dummyKey_;
};

// -----------------------------------------------------------------------------

}  // namespace qg

#endif  // QG_MODEL_OBSBIASINCREMENT_H_
