/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_OBSERR_STREAMQG_H
#define QG_OBSERR_STREAMQG_H

#include <sstream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "QgTraits.h"
#include "oops/interface/ObsErrorBase.h"

namespace eckit {
  class Configuration;
}

namespace qg {

// -----------------------------------------------------------------------------
/// Diagonal observation error covariance matrix.

class ObsErrStreamQG : public oops::ObsErrorBase<QgTraits> {

 public:
  ObsErrStreamQG(const ObsSpaceQG &, const eckit::Configuration &);
  ~ObsErrStreamQG();

/// Linearize and reset for inner loop (nothing in this simple case)
  void linearize(const ObsVecQG &);

/// Setup VarQC weights
  void setupWeights(const ObsVecQG &);

/// Multiply a Departure by \f$R\f$
  ObsVecQG * multiply(const ObsVecQG &) const;

/// Multiply a Departure by \f$R^{-1}\f$
  ObsVecQG * inverseMultiply(const ObsVecQG &) const;

/// Multiply a Departure by \f$W^1/2\f$
  ObsVecQG * multiplyWghtSqrt(const ObsVecQG &) const;

/// Generate random perturbation
  void randomize(ObsVecQG &) const;

/// Get mean error for Jo table
  double getRMSE() const {return stddev_->rms();}

 private:
  void print(std::ostream &) const;

  mutable bool lvarqc_;
  mutable double cleft_;
  mutable double cright_;

  const ObsSpaceQG & obsgeom_;

  boost::scoped_ptr<ObsVecQG> stddev_;
  boost::scoped_ptr<ObsVecQG> inverseVariance_;
  boost::scoped_ptr<ObsVecQG> wghtsqrt_;
};

// =============================================================================

}  // namespace qg

#endif  // QG_OBSERR_STREAMQG_H
