/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/ObsWSpeedQG.h"

#include "eckit/config/Configuration.h"
#include "model/GomQG.h"
#include "model/ObsBias.h"
#include "model/ObsSpaceQG.h"
#include "model/ObsVecQG.h"
#include "model/QgFortran.h"
#include "model/VariablesQG.h"
#include "util/Logger.h"

// -----------------------------------------------------------------------------
namespace qg {
// -----------------------------------------------------------------------------
static oops::ObsOperatorMaker<QgTraits, ObsWSpeedQG> makerWSpeed_("WSpeed");
// -----------------------------------------------------------------------------

ObsWSpeedQG::ObsWSpeedQG(const ObsSpaceQG & odb, const eckit::Configuration & config)
  : keyOperWspeed_(0), varin_()
{
  const eckit::Configuration * configc = &config;
  qg_wspeed_setup_f90(keyOperWspeed_, &configc);
  int keyVarin;
  qg_obsoper_inputs_f90(keyOperWspeed_, keyVarin);
  varin_.reset(new VariablesQG(keyVarin));
  oops::Log::trace() << "ObsWSpeedQG created." << std::endl;
}

// -----------------------------------------------------------------------------

ObsWSpeedQG::~ObsWSpeedQG() {
  qg_wspeed_delete_f90(keyOperWspeed_);
  oops::Log::trace() << "ObsWSpeedQG destructed" << std::endl;
}

// -----------------------------------------------------------------------------

void ObsWSpeedQG::obsEquiv(const GomQG & gom, ObsVecQG & ovec,
                           const boost::ptr_map<std::string, oops::ObsAuxControlBase> & bias) const {
  typedef typename boost::ptr_map<std::string, oops::ObsAuxControlBase>::const_iterator icst_;
  icst_ it = bias.find("ObsBiasQG");
  if (it == bias.end()) {
    double dummyBias = 0.0;
    qg_wspeed_eqv_f90(gom.toFortran(), ovec.toFortran(), dummyBias);
  } else {
    const ObsBias * pbias = dynamic_cast<const ObsBias*>(it->second);
    ASSERT(pbias != nullptr);
    qg_wspeed_eqv_f90(gom.toFortran(), ovec.toFortran(), pbias->wspd());
  }
}

// -----------------------------------------------------------------------------

void ObsWSpeedQG::print(std::ostream & os) const {
  os << "ObsWSpeedQG::print not implemented";
}

// -----------------------------------------------------------------------------

}  // namespace qg
