/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/ObsWSpeedTLAD.h"

#include "eckit/config/Configuration.h"
#include "model/GomQG.h"
#include "model/ObsBias.h"
#include "model/ObsBiasIncrement.h"
#include "model/ObsSpaceQG.h"
#include "model/ObsVecQG.h"
#include "model/QgFortran.h"
#include "model/VariablesQG.h"
#include "util/Logger.h"

// -----------------------------------------------------------------------------
namespace qg {
// -----------------------------------------------------------------------------
static oops::LinearObsOpMaker<QgTraits, ObsWSpeedTLAD> makerWSpeedTL_("WSpeed");
// -----------------------------------------------------------------------------

ObsWSpeedTLAD::ObsWSpeedTLAD(const ObsSpaceQG & odb, const eckit::Configuration & config)
  : keyOperWspeed_(0), traj_(), varin_()
{
  const eckit::Configuration * configc = &config;
  qg_wspeed_setup_f90(keyOperWspeed_, &configc);
  int keyVarin;
  qg_obsoper_inputs_f90(keyOperWspeed_, keyVarin);
  varin_.reset(new VariablesQG(keyVarin));
  qg_wspeed_gettraj_f90(keyOperWspeed_, odb.nobs(), traj_.toFortran());
  oops::Log::trace() << "ObsWSpeedTLAD created" << std::endl;
}

// -----------------------------------------------------------------------------

ObsWSpeedTLAD::~ObsWSpeedTLAD() {
  oops::Log::trace() << "ObsWSpeedTLAD destructed" << std::endl;
}

// -----------------------------------------------------------------------------

void ObsWSpeedTLAD::setTrajectory(const GomQG & gom, 
                                  const boost::ptr_map<std::string, oops::ObsAuxControlBase> &) {
  qg_wspeed_settraj_f90(gom.toFortran(), traj_.toFortran());
  oops::Log::trace() << "ObsWSpeedTLAD trajectory was set " << traj_ << std::endl;
}

// -----------------------------------------------------------------------------

void ObsWSpeedTLAD::obsEquivTL(const GomQG & gom, ObsVecQG & ovec,
                               const boost::ptr_map<std::string, oops::ObsAuxIncrementBase> & bias) const {
  typedef typename boost::ptr_map<std::string, oops::ObsAuxIncrementBase>::const_iterator icst_;
  icst_ it = bias.find("ObsBiasQG");
  if (it == bias.end()) {
    double dummyBias = 0.0;
    qg_wspeed_equiv_tl_f90(gom.toFortran(), ovec.toFortran(),
                           traj_.toFortran(), dummyBias);
  } else {
    ASSERT(it != bias.end());
    const ObsBiasIncrement * pbias = dynamic_cast<const ObsBiasIncrement*>(it->second);
    ASSERT(pbias != nullptr);
    qg_wspeed_equiv_tl_f90(gom.toFortran(), ovec.toFortran(),
                           traj_.toFortran(), pbias->wspd());
  }
}

// -----------------------------------------------------------------------------

void ObsWSpeedTLAD::obsEquivAD(GomQG & gom, const ObsVecQG & ovec,
                               boost::ptr_map<std::string, oops::ObsAuxIncrementBase> & bias) const {
  typedef typename boost::ptr_map<std::string, oops::ObsAuxIncrementBase>::iterator iter_;
  iter_ it = bias.find("ObsBiasQG");
  if (it == bias.end()) {
    double dummyBias = 0.0;
    qg_wspeed_equiv_ad_f90(gom.toFortran(), ovec.toFortran(),
                           traj_.toFortran(), dummyBias);
  } else {
    ObsBiasIncrement * pbias = dynamic_cast<ObsBiasIncrement*>(it->second);
    ASSERT(pbias != nullptr);
    qg_wspeed_equiv_ad_f90(gom.toFortran(), ovec.toFortran(),
                           traj_.toFortran(), pbias->wspd());
  }
}

// -----------------------------------------------------------------------------

void ObsWSpeedTLAD::print(std::ostream & os) const {
  os << "ObsStreamTLAD::print not implemented" << std::endl;
}
// -----------------------------------------------------------------------------

}  // namespace qg
