/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef QG_MODEL_OBSWSPEEDTLAD_H_
#define QG_MODEL_OBSWSPEEDTLAD_H_

#include <string>

#include <boost/shared_ptr.hpp>

#include "oops/interface/LinearObsOperBase.h"
#include "util/ObjectCounter.h"
#include "model/QgTraits.h"

// Forward declarations
namespace eckit {
  class Configuration;
}

namespace qg {
  class GomQG;
  class ObsBias;
  class ObsBiasIncrement;
  class ObsSpaceQG;
  class ObsVecQG;

// -----------------------------------------------------------------------------

class ObsWSpeedTLAD : public oops::LinearObsOperBase<QgTraits>,
                      private util::ObjectCounter<ObsWSpeedTLAD> {
 public:
  static const std::string classname() {return "qg::ObsWSpeedTLAD";}

  ObsWSpeedTLAD(const ObsSpaceQG &, const eckit::Configuration &);
  virtual ~ObsWSpeedTLAD();

// Obs Operators
  void setTrajectory(const GomQG &, 
                     const boost::ptr_map<std::string, oops::ObsAuxControlBase> &);
  void obsEquivTL(const GomQG &, ObsVecQG &, 
                  const boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const;
  void obsEquivAD(GomQG &, const ObsVecQG &, 
                  boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const;
  void obsBiasEquivTL(const GomQG &, ObsVecQG &,
                      const boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const {}
  void obsBiasEquivAD(const GomQG &, const ObsVecQG &,
                      boost::ptr_map<std::string, oops::ObsAuxIncrementBase> &) const {}

// Other
  boost::shared_ptr<const VariablesQG> variables() const override {return varin_;}

  int & toFortran() {return keyOperWspeed_;}
  const int & toFortran() const {return keyOperWspeed_;}

 private:
  void print(std::ostream &) const override;
  F90hop keyOperWspeed_;
  GomQG traj_;
  boost::shared_ptr<const VariablesQG> varin_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_OBSWSPEEDTLAD_H_
