/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "model/ObsWindQG.h"

#include "eckit/config/Configuration.h"
#include "model/GomQG.h"
#include "model/ObsBias.h"
#include "model/ObsSpaceQG.h"
#include "model/ObsVecQG.h"
#include "model/QgFortran.h"
#include "model/VariablesQG.h"
#include "util/Logger.h"

// -----------------------------------------------------------------------------
namespace qg {
// -----------------------------------------------------------------------------
static oops::ObsOperatorMaker<QgTraits, ObsWindQG>   makerWind_("Wind");
// -----------------------------------------------------------------------------

ObsWindQG::ObsWindQG(const ObsSpaceQG & odb, const eckit::Configuration & config)
  : keyOperWind_(0), varin_()
{
  const eckit::Configuration * configc = &config;
  qg_wind_setup_f90(keyOperWind_, &configc);
  int keyVarin;
  qg_obsoper_inputs_f90(keyOperWind_, keyVarin);
  varin_.reset(new VariablesQG(keyVarin));
  oops::Log::trace() << "ObsWindQG created." << std::endl;
}

// -----------------------------------------------------------------------------

ObsWindQG::~ObsWindQG() {
  qg_wind_delete_f90(keyOperWind_);
}

// -----------------------------------------------------------------------------

void ObsWindQG::obsEquiv(const GomQG & gom, ObsVecQG & ovec,
                         const boost::ptr_map<std::string, oops::ObsAuxControlBase> & bias) const {
  typedef typename boost::ptr_map<std::string, oops::ObsAuxControlBase>::const_iterator icst_;
  icst_ it = bias.find("ObsBiasQG");
  if (it == bias.end()) {
    std::vector<double> dummyBias(2, 0.0);
    qg_wind_equiv_f90(gom.toFortran(), ovec.toFortran(), dummyBias[0]);
  } else {
    const ObsBias * pbias = dynamic_cast<const ObsBias*>(it->second);    
    ASSERT(pbias != nullptr);
    qg_wind_equiv_f90(gom.toFortran(), ovec.toFortran(), pbias->wind());
  }
}

// -----------------------------------------------------------------------------

void ObsWindQG::obsBiasEquiv(const GomQG & gom, ObsVecQG & obias,
                         const boost::ptr_map<std::string, oops::ObsAuxControlBase> & bias) const {
  typedef typename boost::ptr_map<std::string, oops::ObsAuxControlBase>::const_iterator icst_;
  icst_ it = bias.find("ObsBiasQG");
  if (it == bias.end()) {
    std::vector<double> dummyBias(2, 0.0);
    qg_wind_bias_equiv_f90(gom.toFortran(), obias.toFortran(), dummyBias[0]);
  } else {
    const ObsBias * pbias = dynamic_cast<const ObsBias*>(it->second);
    ASSERT(pbias != nullptr);
    qg_wind_bias_equiv_f90(gom.toFortran(), obias.toFortran(), pbias->wind());
  }
}

// -----------------------------------------------------------------------------

void ObsWindQG::print(std::ostream & os) const {
  os << "ObsWindQG::print not implemented";
}

// -----------------------------------------------------------------------------

}  // namespace qg
