! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Fortran module handling geometry for the QG model

module qg_geom_mod

use iso_c_binding
use config_mod
use pm_linked_list_mod

implicit none
private
public :: qg_geom
public :: get_geom

! ------------------------------------------------------------------------------

!> Fortran derived type to hold geometry data for the QG model
type :: qg_geom
  integer :: nx
  integer :: ny
end type qg_geom

!> Global registry
type(pm_linked_list) :: geom_list

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine c_qg_geo_setup(c_key_self, c_conf) bind(c,name='qg_geo_setup_f90')
implicit none
integer(c_int), intent(inout) :: c_key_self
type(c_ptr), intent(in)    :: c_conf

type(qg_geom)          :: template
type(qg_geom), pointer :: self

call geom_list%add(c_key_self,template)
self => get_geom(c_key_self)

self%nx = config_get_int(c_conf, "nx")
self%ny = config_get_int(c_conf, "ny")

end subroutine c_qg_geo_setup

! ------------------------------------------------------------------------------

subroutine c_qg_geo_clone(c_key_self, c_key_other) bind(c,name='qg_geo_clone_f90')
implicit none
integer(c_int), intent(in   ) :: c_key_self
integer(c_int), intent(inout) :: c_key_other

type(qg_geom)          :: template 
type(qg_geom), pointer :: self, other

call geom_list%add(c_key_other,template)
other => get_geom(c_key_other)
self => get_geom(c_key_self)

other%nx = self%nx
other%ny = self%ny

end subroutine c_qg_geo_clone

! ------------------------------------------------------------------------------

subroutine c_qg_geo_delete(c_key_self) bind(c,name='qg_geo_delete_f90')

implicit none
integer(c_int), intent(inout) :: c_key_self     

call geom_list%remove(c_key_self)

end subroutine c_qg_geo_delete

! ------------------------------------------------------------------------------

subroutine c_qg_geo_info(c_key_self, c_nx, c_ny) bind(c,name='qg_geo_info_f90')
implicit none
integer(c_int), intent(in   ) :: c_key_self
integer(c_int), intent(inout) :: c_nx
integer(c_int), intent(inout) :: c_ny
type(qg_geom), pointer :: self

self => get_geom(c_key_self)

c_nx = self%nx
c_ny = self%ny

end subroutine c_qg_geo_info

! ------------------------------------------------------------------------------

function get_geom(c_self)
integer(kind=c_int), intent(in) :: c_self
type(qg_geom), pointer :: get_geom
class(*), pointer :: this=>null()

call geom_list%get(c_self, this)

select type(self => this)
type is (qg_geom)
  get_geom=>self
class default
  call abor1_ftn('get_geom : unexpected type ')
end select

end function get_geom

! ------------------------------------------------------------------------------
end module qg_geom_mod
