! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

module QGLOCALIZATION
use iso_c_binding
use qg_fields
use kinds
use qg_constants
use qg_geom_mod
use config_mod
use qg_covariance_mod
use qg_generic_ctl_vec_mod

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_localization_mult(c_key_conf, c_key_xincr) bind(c,name='qg_localization_mult_f90')
implicit none
integer(c_int), intent(in) :: c_key_conf
integer(c_int), intent(in) :: c_key_xincr

type(qg_3d_covar_config), pointer :: conf   !< Config structure
type(qg_field), pointer :: xincr
real(kind=kind_real), allocatable :: xctl(:,:,:) ! Control vector

call qg_3d_cov_registry%get(c_key_conf,conf)
call qg_field_registry%get(c_key_xincr,xincr)

allocate(xctl(conf%nx, conf%ny, 2))

xctl(:,:,:)=0.0_kind_real
call qg_3d_covar_sqrt_mult_ad(conf%nx,conf%ny,xincr,xctl,conf)
call zeros(xincr)
call qg_3d_covar_sqrt_mult(conf%nx,conf%ny,xincr,xctl,conf)

deallocate(xctl)
end subroutine qg_localization_mult

! ------------------------------------------------------------------------------

subroutine qg_localization_mult_sqrt(c_key_conf, c_key_ctlvec_in, c_key_inc_out) &
 & bind(c,name='qg_localization_mult_sqrt_f90')
implicit none
integer(c_int), intent(in) :: c_key_conf         !< covar config structure
integer(c_int), intent(in) :: c_key_ctlvec_in    !< control vector
integer(c_int), intent(in) :: c_key_inc_out      !< increment
type(qg_3d_covar_config), pointer :: conf
type(qg_generic_ctl_vec), pointer :: vin
type(qg_field), pointer :: xout

call qg_3d_cov_registry%get(c_key_conf,conf)
call qg_generic_ctl_vec_registry%get(c_key_ctlvec_in,vin)
call qg_field_registry%get(c_key_inc_out,xout)

call zeros(xout)
call qg_3d_covar_sqrt_mult(conf%nx,conf%ny,xout,vin%ctlvec,conf)

end subroutine qg_localization_mult_sqrt

! ------------------------------------------------------------------------------

subroutine qg_localization_mult_sqrt_trans(c_key_conf, c_key_inc_in, c_key_ctlvec_out) &
 & bind(c,name='qg_localization_mult_sqrt_trans_f90')
implicit none
integer(c_int), intent(in) :: c_key_conf         !< covar config structure
integer(c_int), intent(in) :: c_key_inc_in       !< increment
integer(c_int), intent(in) :: c_key_ctlvec_out   !< control vector
type(qg_3d_covar_config), pointer :: conf
type(qg_field), pointer :: xin
type(qg_generic_ctl_vec), pointer :: vout

call qg_3d_cov_registry%get(c_key_conf,conf)
call qg_field_registry%get(c_key_inc_in,xin)
call qg_generic_ctl_vec_registry%get(c_key_ctlvec_out,vout)

vout%ctlvec(:,:,:)=0.0_kind_real
call qg_3d_covar_sqrt_mult_ad(conf%nx,conf%ny,xin,vout%ctlvec,conf)

end subroutine qg_localization_mult_sqrt_trans

! ------------------------------------------------------------------------------

subroutine qg_localization_setup(c_key_conf, c_model, c_key_geom) bind(c,name='qg_localization_setup_f90')
implicit none
integer(c_int), intent(inout) :: c_key_conf
type(c_ptr), intent(in)    :: c_model  !< The configuration
integer(c_int), intent(in) :: c_key_geom !< Geometry
type(qg_3d_covar_config), pointer :: conf !< covar structure
type(qg_geom), pointer :: geom     !< Geometry

call qg_3d_cov_registry%init()
call qg_3d_cov_registry%add(c_key_conf)
call qg_3d_cov_registry%get(c_key_conf, conf)
geom => get_geom(c_key_geom)

call qg_3d_covar_setup(c_model, geom, conf)

return
end subroutine qg_localization_setup

! ------------------------------------------------------------------------------

subroutine qg_localization_delete(c_key_conf) bind(c,name='qg_localization_delete_f90')
implicit none
integer(c_int), intent(inout) :: c_key_conf !< The model covariance structure

call qg_3d_covar_delete(c_key_conf)

end subroutine qg_localization_delete

! ------------------------------------------------------------------------------

!> Get dimensions for the control vector

subroutine c_qg_localization_get_dim(c_key_conf, c_dim) &
          & bind (c,name='qg_localization_get_dim_f90')

use iso_c_binding
use qg_covariance_mod

implicit none
integer(c_int), intent(in) :: c_key_conf   !< The background covariance structure
integer(c_int), intent(inout) :: c_dim(3)  !< Dimensions
type(qg_3d_covar_config), pointer :: conf

call qg_3d_cov_registry%get(c_key_conf, conf)

c_dim(1) = conf%nx
c_dim(2) = conf%ny
c_dim(3) = 2

end subroutine c_qg_localization_get_dim

! ------------------------------------------------------------------------------


end module QGLOCALIZATION
