! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Fortran module handling observation locations

module qg_locs_mod

use iso_c_binding
use qg_obs_vectors
use kinds
use pm_linked_list_mod

implicit none
private
public :: qg_locs, qg_loc_setup
public :: get_locs, locs_list

! ------------------------------------------------------------------------------

!> Fortran derived type to hold observation locations
type :: qg_locs
  integer :: nloc
  real(kind=kind_real), allocatable :: xyz(:,:)
end type qg_locs

!> Global registry
type(pm_linked_list) :: locs_list

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_loc_setup(self, lvec)
implicit none
type(qg_locs), intent(inout) :: self
type(obs_vect), intent(in) :: lvec
integer :: jc, jo

self%nloc=lvec%nobs
allocate(self%xyz(3,self%nloc))
do jo=1,self%nloc
  do jc=1,3
    self%xyz(jc,jo)=lvec%values(jc,jo)
  enddo
enddo

end subroutine qg_loc_setup

! ------------------------------------------------------------------------------

subroutine c_qg_loc_delete(key) bind(c,name='qg_loc_delete_f90')

implicit none
integer(c_int), intent(inout) :: key
type(qg_locs), pointer :: self

self => get_locs(key)

deallocate(self%xyz)
call locs_list%remove(key)

end subroutine c_qg_loc_delete

! ------------------------------------------------------------------------------

subroutine c_qg_loc_nobs(key, kobs) bind(c,name='qg_loc_nobs_f90')

implicit none
integer(c_int), intent(in) :: key
integer(c_int), intent(inout) :: kobs
type(qg_locs), pointer :: self

self => get_locs(key)

kobs = self%nloc

end subroutine c_qg_loc_nobs

! ------------------------------------------------------------------------------

function get_locs(c_self)
integer(kind=c_int), intent(in) :: c_self
type(qg_locs), pointer :: get_locs
class(*), pointer :: this=>null()

call locs_list%get(c_self, this)

select type(self => this)
type is (qg_locs)
  get_locs=>self
class default
  call abor1_ftn('get_locs : unexpected type ')
end select

end function get_locs

! ------------------------------------------------------------------------------

end module qg_locs_mod
