! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Fortran module for streamfunction observations for the QG model
module qg_weights_mod

use iso_c_binding
!use config_mod
!use duration_mod
!use qg_obs_data
use qg_obs_vectors
!use qg_vars_mod
use kinds

implicit none
private

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_obserr_setup_wghts_sqrt(c_key_self, c_key_stddev, c_key_dy,   &
 &                                    c_cleft, c_cright                   ) &
 &                            bind(c,name='qg_obserr_setup_wghts_sqrt_f90')
implicit none
integer(c_int), intent(in) :: c_key_self
integer(c_int), intent(in) :: c_key_stddev
integer(c_int), intent(in) :: c_key_dy
real(c_double), intent(inout) :: c_cleft
real(c_double), intent(inout) :: c_cright
type(obs_vect), pointer :: self, stddev, dy
real(c_double) :: rhoNorm, rhoHuber
integer :: jc, jo

self => get_obs_vect(c_key_self)
stddev => get_obs_vect(c_key_stddev)
dy => get_obs_vect(c_key_dy)

! Setup W^1/2

self%values(:,:) = 1.0

do jo=1,self%nobs
do jc=1,self%ncol
  if (dy%values(jc,jo) < c_cleft .or. dy%values(jc,jo) > c_cright) then
    ! rho(x) = x^2/sigma^2          if |x|<=c
    ! rho(x) = (2c|x|-c^2)/sigma^2  if |x|>c
    ! W(x)   = J_QC/J_N
    rhoNorm = dy%values(jc,jo)**2/stddev%values(jc,jo)**2
    if (dy%values(jc,jo) < c_cleft ) then
      rhoHuber = (2*dabs(c_cleft*dy%values(jc,jo))-c_cleft**2)/stddev%values(jc,jo)**2
      self%values(jc,jo) = rhoHuber/rhoNorm
    elseif(dy%values(jc,jo) > c_cright ) then
      rhoHuber = (2*c_cright*dy%values(jc,jo)-c_cright**2)/stddev%values(jc,jo)**2
      self%values(jc,jo) = rhoHuber/rhoNorm
    endif
  endif
enddo
enddo

self%values(:,:) = sqrt(self%values(:,:))

end subroutine qg_obserr_setup_wghts_sqrt 

! ------------------------------------------------------------------------------

end module qg_weights_mod
