/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef OOPS_ASSIMILATION_SYMMATSPECTRUM_H_
#define OOPS_ASSIMILATION_SYMMATSPECTRUM_H_

#include <vector>

#include "util/Logger.h"

extern "C" {
  void FtnSymMatSpectrum(const int &, const double *, double *, double *);
}

namespace oops {

void SymMatSpectrum(const std::vector<double> & mat,
                    std::vector<double> & evals,
                    std::vector<std::vector<double> > & evecs) {
  const int nn2 = mat.size();
  const int nn = sqrt(nn2);
  ASSERT(nn*nn == nn2);
  evals.resize(nn);
  evecs.resize(nn);

  if (nn > 0) {
    std::vector<double> ftmat(nn*nn);
    std::vector<double> ftnev(nn*nn);

    ftmat = mat;

    FtnSymMatSpectrum(nn, &ftmat[0], &evals[0], &ftnev[0]);

    size_t ii = 0;
    for (int jv = 0; jv < nn; ++jv) {
      evecs[jv].resize(nn);
      for (int jj = 0; jj < nn; ++jj) {
        evecs[jv][jj] = ftnev[ii];
        ++ii;
      }
    }
  }
}

}  // namespace oops

#endif  // OOPS_ASSIMILATION_SYMMATSPECTRUM_H_
