/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef OOPS_INTERFACE_OBSAUXCONTROLBASE_H_
#define OOPS_INTERFACE_OBSAUXCONTROLBASE_H_

#include <iostream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "util/abor1_cpp.h"
#include "util/Logger.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"
#include "util/Timer.h"

namespace eckit {
  class Configuration;
}

namespace oops {

  class ObsAuxIncrementBase;

// -----------------------------------------------------------------------------

class ObsAuxControlBase : public util::Printable {
 public:
/// Constructor, destructor
  ObsAuxControlBase() {}
  virtual ~ObsAuxControlBase() {}

/// Cloning
  virtual ObsAuxControlBase* clone(const bool copy = true) const =0;

/// Operators
  virtual ObsAuxControlBase & operator+=(const ObsAuxIncrementBase &) =0;

/// Reset
  virtual void reset(const std::string &, const eckit::Configuration &) =0;

/// I/O and diagnostics
  virtual void read(const eckit::Configuration &) =0;
  virtual void write(const eckit::Configuration &) const =0;
  virtual double norm() const =0;

/// Interfacing
  virtual const int & getAccess() const =0;
  virtual int & getAccess() =0;

 private:
  virtual void print(std::ostream &) const =0;
};

// =============================================================================

/// ObsAuxControl Factory
class ObsAuxControlFactory {
 public:
  static ObsAuxControlBase * create(const eckit::Configuration &);
  virtual ~ObsAuxControlFactory() { getMakers().clear(); }
 protected:
  explicit ObsAuxControlFactory(const std::string &);
 private:
  virtual ObsAuxControlBase * make(const eckit::Configuration &) =0;
  static std::map < std::string, ObsAuxControlFactory * > & getMakers() {
    static std::map < std::string, ObsAuxControlFactory * > makers_;
    return makers_;
  }
};

// -----------------------------------------------------------------------------

template<class T>
class ObsAuxControlMaker : public ObsAuxControlFactory {
  virtual ObsAuxControlBase * make(const eckit::Configuration & conf)
    { return new T(conf); }
 public:
  explicit ObsAuxControlMaker(const std::string & name) : ObsAuxControlFactory(name) {}
};

// =============================================================================

}  // namespace oops

#endif  // OOPS_INTERFACE_OBSAUXCONTROLBASE_H_
