/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "oops/interface/ObsAuxCovarianceBase.h"

namespace oops {

// =============================================================================

ObsAuxCovarianceFactory::ObsAuxCovarianceFactory(const std::string & name) {
  if (getMakers().find(name) != getMakers().end()) {
    Log::error() << name << " already registered in ObsAuxCovarianceFactory." << std::endl;
    ABORT("Element already registered in ObsAuxCovarianceFactory.");
  }
  getMakers()[name] = this;
}

// -----------------------------------------------------------------------------

ObsAuxCovarianceBase* ObsAuxCovarianceFactory::create(const eckit::Configuration & conf) {
  const std::string id = conf.getString("type");
  typename std::map<std::string, ObsAuxCovarianceFactory*>::iterator
    jloc = getMakers().find(id);
  if (jloc == getMakers().end()) {
    Log::error() << id << " does not exist in ObsAuxCovarianceFactory." << std::endl;
    ABORT("Element does not exist in ObsAuxCovarianceFactory.");
  }
  ObsAuxCovarianceBase * ptr = jloc->second->make(conf);
  return ptr;
}

// =============================================================================

}  // namespace oops
