/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef OOPS_INTERFACE_OBSAUXCOVARIANCEBASE_H_
#define OOPS_INTERFACE_OBSAUXCOVARIANCEBASE_H_

#include <iostream>
#include <string>

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "eckit/config/Configuration.h"

#include "util/abor1_cpp.h"
#include "util/Logger.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"
#include "util/Timer.h"

namespace oops {

class ObsAuxControlBase;
class ObsAuxCtlVecBase;
class ObsAuxIncrementBase;

// -----------------------------------------------------------------------------

class ObsAuxCovarianceBase : public util::Printable,
                             private boost::noncopyable {
  typedef ObsAuxControlBase   ObsAuxControlBase_;
  typedef ObsAuxCtlVecBase    ObsAuxCtlVecBase_;
  typedef ObsAuxIncrementBase ObsAuxIncrementBase_;

 public:
  static const std::string classname() {return "oops::ObsAuxCovariance";}

  ObsAuxCovarianceBase() {}
  virtual ~ObsAuxCovarianceBase() {}

/// Update
  virtual void update(const ObsAuxControlBase_ &, const eckit::Configuration &) =0;

/// Operators
  virtual void linearize(const ObsAuxControlBase_ &) =0;
  virtual void multiply(const ObsAuxIncrementBase_ &, ObsAuxIncrementBase_ &) const =0;
  virtual void inverseMultiply(const ObsAuxIncrementBase_ &, ObsAuxIncrementBase_ &) const =0;
  virtual void multiplySqrt(const ObsAuxCtlVecBase_ &, ObsAuxIncrementBase_ &) const =0;
  virtual void multiplySqrtTrans(const ObsAuxIncrementBase_ &, ObsAuxCtlVecBase_ &) const =0;
  virtual void randomize(ObsAuxIncrementBase_ &) const =0;

// Interfacing
  virtual const eckit::Configuration & config() const =0;
  virtual const int & getAccess() const =0;
  virtual int & getAccess() =0;

 private:
  virtual void print(std::ostream &) const =0;
};

// =============================================================================

/// ObsAuxCovariance Factory
class ObsAuxCovarianceFactory {
 public:
  static ObsAuxCovarianceBase * create(const eckit::Configuration &);
  virtual ~ObsAuxCovarianceFactory() { getMakers().clear(); }
 protected:
  explicit ObsAuxCovarianceFactory(const std::string &);
 private:
  virtual ObsAuxCovarianceBase * make(const eckit::Configuration &) =0;
  static std::map < std::string, ObsAuxCovarianceFactory * > & getMakers() {
    static std::map < std::string, ObsAuxCovarianceFactory * > makers_;
    return makers_;
  }
};

// -----------------------------------------------------------------------------

template<class T>
class ObsAuxCovarianceMaker : public ObsAuxCovarianceFactory {
  virtual ObsAuxCovarianceBase * make(const eckit::Configuration & conf)
    { return new T(conf); }
 public:
  explicit ObsAuxCovarianceMaker(const std::string & name) : ObsAuxCovarianceFactory(name) {}
};

// =============================================================================

}  // namespace oops

#endif  // OOPS_INTERFACE_OBSAUXCOVARIANCEBASE_H_
