/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "oops/interface/ObsAuxCtlVecBase.h"
#include "oops/interface/ObsAuxCovarianceBase.h"

namespace oops {

// =============================================================================

ObsAuxCtlVecFactory::ObsAuxCtlVecFactory(const std::string & name) {
  if (getMakers().find(name) != getMakers().end()) {
    Log::error() << name << " already registered in ObsAuxCtlVecFactory." << std::endl;
    ABORT("Element already registered in ObsAuxCtlVecFactory.");
  }
  getMakers()[name] = this;
}

// -----------------------------------------------------------------------------

ObsAuxCtlVecBase* ObsAuxCtlVecFactory::create(const ObsAuxCovarianceBase_ & cov) {
  const std::string id = cov.config().getString("type");
  typename std::map<std::string, ObsAuxCtlVecFactory*>::iterator
    jloc = getMakers().find(id);
  if (jloc == getMakers().end()) {
    Log::error() << id << " does not exist in ObsAuxCtlVecFactory." << std::endl;
    ABORT("Element does not exist in ObsAuxCtlVecFactory.");
  }
  ObsAuxCtlVecBase * ptr = jloc->second->make(cov);
  return ptr;
}

// =============================================================================

}  // namespace oops
