/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef OOPS_INTERFACE_OBSAUXINCREMENTBASE_H_
#define OOPS_INTERFACE_OBSAUXINCREMENTBASE_H_

#include <iostream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "util/abor1_cpp.h"
#include "util/Logger.h"
#include "util/ObjectCounter.h"
#include "util/Printable.h"
#include "util/Timer.h"

namespace eckit {
  class Configuration;
}

namespace oops {

class ObsAuxControlBase;

// -----------------------------------------------------------------------------

class ObsAuxIncrementBase : public util::Printable {
  typedef ObsAuxControlBase ObsAuxControlBase_;

 public:
  static const std::string classname() {return "oops::ObsAuxIncrementBase";}

/// Constructor, destructor
  ObsAuxIncrementBase() {}
  virtual ~ObsAuxIncrementBase() {}

/// Cloning
  virtual ObsAuxIncrementBase* clone(const bool copy = true) const =0;
  virtual ObsAuxIncrementBase* clone(const eckit::Configuration &) const =0;

/// Linear algebra operators
  virtual void diff(const ObsAuxControlBase_ &, const ObsAuxControlBase_ &) =0;
  virtual void zero() =0;
  virtual ObsAuxIncrementBase & operator=(const ObsAuxIncrementBase &) =0;
  virtual ObsAuxIncrementBase & operator+=(const ObsAuxIncrementBase &) =0;
  virtual ObsAuxIncrementBase & operator-=(const ObsAuxIncrementBase &) =0;
  virtual ObsAuxIncrementBase & operator*=(const double &) =0;
  virtual void axpy(const double &, const ObsAuxIncrementBase &) =0;
  virtual double dot_product_with(const ObsAuxIncrementBase &) const =0;

/// virtual / I/O and diagnostics
  virtual void read(const eckit::Configuration &) =0;
  virtual void write(const eckit::Configuration &) const =0;
  virtual double norm() const =0;

/// Interfacing
  virtual const int & getAccess() const =0;
  virtual int & getAccess() =0;

 private:
  virtual void print(std::ostream &) const =0;
};

// -----------------------------------------------------------------------------

/// ObsAuxIncrement Factory
class ObsAuxIncrementFactory {
 public:
  static ObsAuxIncrementBase * create(const eckit::Configuration &);
  virtual ~ObsAuxIncrementFactory() { getMakers().clear(); }
 protected:
  explicit ObsAuxIncrementFactory(const std::string &);
 private:
  virtual ObsAuxIncrementBase * make(const eckit::Configuration &) =0;
  static std::map < std::string, ObsAuxIncrementFactory * > & getMakers() {
    static std::map < std::string, ObsAuxIncrementFactory * > makers_;
    return makers_;
  }
};

// -----------------------------------------------------------------------------

template<class T>
class ObsAuxIncrementMaker : public ObsAuxIncrementFactory {
  virtual ObsAuxIncrementBase * make(const eckit::Configuration & conf)
    { return new T(conf); }
 public:
  explicit ObsAuxIncrementMaker(const std::string & name) : ObsAuxIncrementFactory(name) {}
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_INTERFACE_OBSAUXINCREMENTBASE_H_
