/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef UTIL_LOGBOOK_H_
#define UTIL_LOGBOOK_H_

#include "eckit/config/LocalConfiguration.h"
#include "eckit/config/YAMLConfiguration.h"

namespace util {

// -----------------------------------------------------------------------------

class Logbook : public eckit::LocalConfiguration {
 public:
/// Constructors, destructor
  Logbook() : eckit::LocalConfiguration() {}
  Logbook(const eckit::YAMLConfiguration & conf) : eckit::LocalConfiguration(conf) {}
  Logbook(const eckit::LocalConfiguration & conf) : eckit::LocalConfiguration(conf) {}
  Logbook(const Logbook & logbook) : eckit::LocalConfiguration(logbook) {}

/// Accessors
  template<typename ValueT>
  void get(const std::string & name, ValueT & value) const {
    if (not eckit::LocalConfiguration::get(name, value)) throw_exception(name);
  }

  template<typename ValueT>
  void set(const std::string & name, const ValueT & value) {
    eckit::LocalConfiguration::set(name, value);
  }

  bool has(const std::string & name) const {
    return eckit::LocalConfiguration::has(name);
  }

/// Communication
  void broadcast(const size_t root =0);
 private:
  Logbook(const eckit::Value & value) : eckit::LocalConfiguration(value) {}
  void broadcast(Logbook & logbook, const size_t root);
  void throw_exception(const std::string & name) const;
};

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_LOGBOOK_H_
