/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "util/LogbookHandler.h"

#include <map>
#include <ostream>

#include "util/Logger.h"
#include "util/Mpi.h"

#include "eckit/config/YAMLConfiguration.h"
#include "eckit/exception/Exceptions.h"
#include "eckit/filesystem/LocalPathName.h"
#include "eckit/parser/JSON.h"
#include "eckit/parser/JSONParser.h"

namespace util {

// -----------------------------------------------------------------------------

LogbookHandler::LogbookHandler(const eckit::PathName & path, const size_t root) 
 : root_(root), path_(path), logbook_()
{
  if (path_.exists()) {
    oops::Log::info() << "LogbookHandler::LogbookHandler Path : "  
                      << path_ << " exists." << std::endl;
    if (mpi::comm().rank() == root_) {
      eckit::YAMLConfiguration yaml(path_);
      logbook_.reset(new Logbook(yaml));
    } else {
      logbook_.reset(new Logbook());
    }
    logbook_->broadcast();
  } else { 
    oops::Log::info() << "LogbookHandler::LogbookHandler Path : "  
                      << path_ << " does not exist." << std::endl;
    logbook_.reset(new Logbook());
  }
}

// -----------------------------------------------------------------------------

void LogbookHandler::broadcast() {
  oops::Log::info() << "LogbookHandler::broadcast distributing the Logbook" << std::endl;
  logbook_->broadcast(root_);
}

// -----------------------------------------------------------------------------

void LogbookHandler::write() const {
  if (mpi::comm().rank() == root_) {
    eckit::PathName uniquepath = eckit::PathName::unique(path_);
    std::ofstream ofs(uniquepath.localPath());
    if (!ofs)
      throw eckit::CantOpenFile(uniquepath.localPath(), Here());
    eckit::JSON j(ofs);
    j << *logbook_;
    ofs.close();
    eckit::PathName::rename(uniquepath, path_);
  }
} 

// -----------------------------------------------------------------------------

void LogbookHandler::print(std::ostream & os) const {
  os << *logbook_;
}

// -----------------------------------------------------------------------------

}  // namespace util
