/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef UTIL_LOGBOOKHANDLER_H_
#define UTIL_LOGBOOKHANDLER_H_

#include <string>

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "util/Logbook.h"
#include "util/Printable.h"

#include "eckit/filesystem/PathName.h"
#include "eckit/memory/ScopedPtr.h"

namespace util {

// -----------------------------------------------------------------------------

class LogbookHandler : public util::Printable,
                     private boost::noncopyable {
 public:
/// Constructor, destructor
  explicit LogbookHandler(const eckit::PathName & path, const size_t root =0); 

/// Accessors
  template<typename ValueT>
  void get(const std::string & name, ValueT & value) const {
    logbook_->get<ValueT>(name, value);
  }
  template<typename ValueT>
  void set(const std::string & name, const ValueT & value) {
    logbook_->set<ValueT>(name, value);
  }
  bool has(const std::string & name) const {
    return logbook_->has(name);
  }

/// I/O
  void broadcast();
  void write() const;

 private:
  void print(std::ostream & os) const;

  const size_t root_;
  const eckit::PathName path_;
  boost::scoped_ptr<Logbook> logbook_;
};

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_LOGBOOKHANDLER_H_
