/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "util/LogbookHelper.h"

#include <iostream>

#include "eckit/config/LocalConfiguration.h"
#include "eckit/filesystem/LocalPathName.h"

using oops::Log;

namespace util {

// -----------------------------------------------------------------------------

LogbookHelper & LogbookHelper::getInstance() {
  static LogbookHelper theHelper;
  return theHelper;
}

// -----------------------------------------------------------------------------

void LogbookHelper::start() {
  oops::Log::info() << "LogbookHelper::start() Start Logbook recording" << std::endl;
  getInstance().logbook_.reset(new Logbook());
  getInstance().on_ = true;
  getInstance().counts_.clear();
}

// -----------------------------------------------------------------------------

void LogbookHelper::stop() {
  Log::info() << getInstance() << std::endl;
  getInstance().logbook_.reset();
  getInstance().counts_.clear();
  getInstance().status_.clear();
  getInstance().on_ = false;
}

// -----------------------------------------------------------------------------

void LogbookHelper::finishRecording(const std::string & name) {
  if (getInstance().on_) {
    getInstance().counts_[name] += 1;
    getInstance().status_[name] = Execution::NotActive;
    getInstance().record<bool>(name+"_active", Execution::NotActive, Callback::Call);
  }
}

// -----------------------------------------------------------------------------

bool LogbookHelper::has(const std::string & s) {
  ASSERT(getInstance().logbook_);
  return getInstance().logbook_->has(s);
}

// -----------------------------------------------------------------------------

void LogbookHelper::connectCallback(const CallbackFunction_ cb) {
  ASSERT(getInstance().logbook_);
  getInstance().callbacks_.push_back(cb);
}

// -----------------------------------------------------------------------------

const eckit::Configuration & LogbookHelper::getConstLogbook() {
  ASSERT(getInstance().logbook_);
  return *(getInstance().logbook_);
} 

// -----------------------------------------------------------------------------

eckit::LocalConfiguration & LogbookHelper::getLogbook() {
  ASSERT(getInstance().logbook_);
  return *(getInstance().logbook_);
}

// -----------------------------------------------------------------------------

LogbookHelper::LogbookHelper()
  : on_(false), status_(), counts_(), logbook_() {} 

// -----------------------------------------------------------------------------

LogbookHelper::~LogbookHelper() {}

// -----------------------------------------------------------------------------

void LogbookHelper::print(std::ostream & os) const {
  os << " " << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << "------------------------- Logbook Helper Content --------------------" << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  for (cit_ jt = counts_.begin(); jt != counts_.end(); ++jt) {
    os << std::setw(52) << std::left << jt->first
       << ": " << std::setw(12) << std::right << jt->second << std::endl;
  }
  os << "------------------------- Logbook Helper Content End ----------------" << std::endl;
  os << " " << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << "------------------------- Logbook Content ---------------------------" << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << *logbook_ << std::endl;
  os << "------------------------- Logbook Content End -----------------------" << std::endl;
}

// -----------------------------------------------------------------------------

}  // namespace util
