/*
 * (C) Copyright 2009-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef UTIL_LOGBOOKWRITER_H_
#define UTIL_LOGBOOKWRITER_H_

#include <string>

#include <boost/noncopyable.hpp>

#include "util/LogbookHelper.h"

namespace util {

// -----------------------------------------------------------------------------

template <typename ValueT>
class LogbookWriter : private boost::noncopyable {
 public:
/// Contructor, destructor
  LogbookWriter(const std::string &, const ValueT &);
  ~LogbookWriter();

 private:
  const std::string key_;
};

// -----------------------------------------------------------------------------

template<typename ValueT>
LogbookWriter<ValueT>::LogbookWriter(const std::string & key, const ValueT & val)
  : key_(key)
{
  LogbookHelper::startRecording<ValueT>(key, val);
}

// -----------------------------------------------------------------------------

template<typename ValueT>
LogbookWriter<ValueT>::~LogbookWriter() {
  LogbookHelper::finishRecording(key_);
};

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_LOGBOOKWRITER_H_
