/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#include "util/RestartHelper.h"

#include <iostream>

#include "eckit/config/LocalConfiguration.h"
#include "eckit/filesystem/LocalPathName.h"

using oops::Log;

namespace util {

// -----------------------------------------------------------------------------

RestartHelper & RestartHelper::getInstance() {
  static RestartHelper theHelper;
  return theHelper;
}

// -----------------------------------------------------------------------------

void RestartHelper::start() {
  oops::Log::info() << "RestartHelper::start() Start Restart Logbook recording" << std::endl;

  if (getInstance().path_.exists()) 
    getInstance().isReadIn_ = true;
  getInstance().logbookHandler_.reset(new LogbookHandler(getInstance().path_));
  getInstance().on_ = true;

  oops::Log::info() << *getInstance().logbookHandler_ << std::endl;
}

// -----------------------------------------------------------------------------

void RestartHelper::stop() {
  Log::info() << getInstance() << std::endl;
  getInstance().logbookHandler_.reset();
  getInstance().on_ = false;
}

// -----------------------------------------------------------------------------

void RestartHelper::write() {
  getInstance().logbookHandler_->write();
}

// -----------------------------------------------------------------------------

bool RestartHelper::has(const std::string & s) {
  ASSERT(getInstance().logbookHandler_);
  return getInstance().logbookHandler_->has(s);
}

// -----------------------------------------------------------------------------

const bool & RestartHelper::isReadIn() {
  ASSERT(getInstance().on_);
  return getInstance().isReadIn_;
}

// -----------------------------------------------------------------------------

RestartHelper::RestartHelper()
  : on_(false), isReadIn_(false), path_(), logbookHandler_() 
{
  eckit::PathName path = eckit::LocalPathName::cwd() + "/logbook.json";
  path_ = path;
}

// -----------------------------------------------------------------------------

RestartHelper::~RestartHelper() {}

// -----------------------------------------------------------------------------

void RestartHelper::print(std::ostream & os) const {
  os << " " << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << "------------------------- Restart Content ---------------------------" << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << "Restart Logbook path" << path_ << std::endl;
  os << "---------------------------------------------------------------------" << std::endl;
  os << *logbookHandler_ << std::endl;
  os << "------------------------- Restart Content End -----------------------" << std::endl;
}

// -----------------------------------------------------------------------------

}  // namespace util
