/*
 * (C) Copyright 2009-2016 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

#ifndef UTIL_RESTARTHELPER_H_
#define UTIL_RESTARTHELPER_H_

#include <map>
#include <string>

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "util/LogbookHandler.h"
#include "util/Logger.h"
#include "util/Printable.h"

#include "eckit/config/LocalConfiguration.h"
#include "eckit/exception/Exceptions.h"
#include "eckit/filesystem/PathName.h"

namespace util {

// -----------------------------------------------------------------------------

class RestartHelper : public util::Printable,
                      private boost::noncopyable {
 public:
/// Initialization, finalization
  static void start();
  static void stop();

/// Utilities
  static void write();
  static const bool & isReadIn();

/// Accessors
  static bool has(const std::string & s);
  template<typename ValueT> static void query(const std::string & s,
                                              ValueT & v);
  template<typename ValueT> static void record(const std::string & s, 
                                               const ValueT & v, 
                                               const bool writeToDisc = false);

  ~RestartHelper();
 private:
  RestartHelper();
  static RestartHelper & getInstance();
  void print(std::ostream & os) const;

  bool on_;
  mutable bool isReadIn_;
  mutable eckit::PathName path_;
  boost::scoped_ptr<LogbookHandler> logbookHandler_;
};

// -----------------------------------------------------------------------------

template<typename ValueT>
void RestartHelper::query(const std::string & s, ValueT & v) {
  ASSERT(getInstance().logbookHandler_);
  return getInstance().logbookHandler_->get<ValueT>(s, v);
}

// -----------------------------------------------------------------------------

template<typename ValueT>
void RestartHelper::record(const std::string & s, const ValueT & v, const bool writeToDisc) {
  ASSERT(getInstance().logbookHandler_);
  getInstance().logbookHandler_->set<ValueT>(s, v);
  if (writeToDisc) getInstance().logbookHandler_->write();
}

// -----------------------------------------------------------------------------

}  // namespace util

#endif  // UTIL_RESTARTHELPER_H_
